require 'uri'
require 'securerandom'
require 'uu/os/application/abstract_controller'
require 'uu/os/server/immediate_response'
require 'uu/os/application/config'
require 'uu/os/cmd/command_context'
require 'uu/os/cmd/command_callback_context'
require 'uu/os/http/http_request'
require 'uu/os/ipc/ipc_error'
require 'uu/os/ipc/error_codes'
require 'uu/os/ipc/error_kind'
require 'uu/os/application/application_error'
require 'uu/os/application/error_codes'
require 'uu/os/security/session'
require 'uu/os/rest/client_exception'
require 'uu/os/rest/remote_exception'
require 'uu/os/logger'
require 'uu/os/util/container_service'

module UU
  module OS
    module CMD

      # Ancestor for development of custom command controllers.
      # All controllers must define +PATH+ constant (for controller
      # mapping) and should define at least {#on_exec on_exec} method.
      # Additionally, it is possible to modify default processing
      # behavior by overriding methods declared in
      # {UU::OS::Application::AbstractController AbstractController}.
      class CommandController < UU::OS::Application::AbstractController

        # Main action method of controller.
        # @param [UU::OS::CMD::CommandContext] ctx Instance of execution
        #    context.
        # @return [Object] Result of command (any object to be serialized
        #    via +to_json+ method, or response provided by {UU::OS::Server::AbstractServlet#response
        #    response}, {UU::OS::Server::AbstractServlet#file_download_response file_download_response}
        #    or {UU::OS::Server::AbstractServlet#streamed_response streamed_response} methods).
        def on_exec(ctx)
          raise UU::OS::CMD::CommandError.new(UU::OS::IPC::ErrorKind::SERVER, UU::OS::IPC::ErrorCodes::NOT_IMPLEMENTED,
          	"Method on_exec is not implemented on #{self.class.name}.")
        end


        # Checks if invocation method (POST, GET, etc.) matches the
        # controller type. By default, "get*" commands should be
        # invoked as GET, any other commands should be invoked as POST.
        # @param [UU::OS::CMD::CommandContext] ctx Instance of execution
        #    context.
        # @return [void]
        # @raise [UU::OS::Application::ControllerError] In case invocation
        #    method does not match the expected one.
        # @note Not intended to be used from controller implementation but for
        #    optional overriding of default behavior by custom implementation!
        # @private
        def assert_invocation_method(ctx) # :nodoc:
          request = ctx.request
          parts = request.url.split('/')
          if parts[-1] =~ /^get/ || parts[-2] =~ /^get/
            raise UU::OS::Application::ApplicationError.new(UU::OS::IPC::ErrorKind::CLIENT, UU::OS::Application::ErrorCodes::INVALID_REQUEST,
              "Invalid invocation method, expected GET or POST.") if request.method != :get && request.method != :post
          else
            raise UU::OS::Application::ApplicationError.new(UU::OS::IPC::ErrorKind::CLIENT, UU::OS::Application::ErrorCodes::INVALID_REQUEST,
              "Invalid invocation method, expected POST.") if request.method != :post
          end
        end

        # @private
        def get_context(request)
          ctx = super(request)
          ctx.instance_variable_set(:@forbid_async, true) unless ctx.nil?
          return ctx
        end

        # Initializes authn environment and evaluates incoming
        # authn token.
        # @private
        def prepare_execution(ctx) # :nodoc:
          super(ctx)
        end

        # Cleans up authn environment.
        # @private
        def finalize_execution(ctx) # :nodoc:
          super(ctx)
        end

      end

    end
  end
end
